/**
 *  Copyright (c) 1997-2013, tinygroup.org (luo_guo@live.cn).
 *
 *  Licensed under the GPL, Version 3.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *       http://www.gnu.org/licenses/gpl.html
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 * --------------------------------------------------------------------------
 *  版权 (c) 1997-2013, tinygroup.org (luo_guo@live.cn).
 *
 *  本开源软件遵循 GPL 3.0 协议;
 *  如果您不遵循此协议，则不被允许使用此文件。
 *  你可以从下面的地址获取完整的协议文本
 *
 *       http://www.gnu.org/licenses/gpl.html
 */
package org.tinygroup.net.daemon;

import org.tinygroup.logger.Logger;
import org.tinygroup.logger.LoggerFactory;

/**
 * 监护线程
 * 
 * @author luoguo
 * 
 */
public abstract class DaemonRunnable implements Runnable {
	private static final int DEFAULT_WAIT_TIME = 5000;
	private int waitTime = DEFAULT_WAIT_TIME;// 如果机器停掉，则等待这段时间之后，再做下一次连接
	private final Logger logger = LoggerFactory.getLogger(this.getClass());
	volatile boolean end = false;

	public void stop() {
		this.end = true;
		stopAction();
	}

	public int getWaitTime() {
		return waitTime;
	}

	public void setWaitTime(int waitTime) {
		this.waitTime = waitTime;
	}

	public void run() {
		while (!end) {
			try {
				System.out.println("开始运行...");
				startAction();
				System.out.println("运行结束。");
			} catch (Throwable e) {
				logger.error(e);
			} finally {
				try {
					Thread.sleep(waitTime);
				} catch (InterruptedException e) {
					logger.error(e);
				}

			}
		}
	}

	/**
	 * 真正要做事情的地方
	 */
	protected abstract void startAction();

	protected abstract void stopAction();
}
